/*
    bsp_rf_driver.c

-----------------------------------------------------------------------------

  This file is part of mikroSDK.

  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

#include "hal.h"
#include "bsp_rf_driver.h"

#if ( _MMB_MODULE_RF == 1 )

/* ------------------------------------------------------------------- MACROS */

/* Memory Map */

#define CONFIG                                  0x00
#define EN_AA                                   0x01
#define EN_RXADDR                               0x02
#define SETUP_AW                                0x03
#define SETUP_RETR                              0x04
#define RF_CH                                   0x05
#define RF_SETUP                                0x06
#define STATUS                                  0x07
#define OBSERVE_TX                              0x08
#define CD                                      0x09
#define RX_ADDR_P0                              0x0A
#define RX_ADDR_P1                              0x0B
#define RX_ADDR_P2                              0x0C
#define RX_ADDR_P3                              0x0D
#define RX_ADDR_P4                              0x0E
#define RX_ADDR_P5                              0x0F
#define TX_ADDR                                 0x10
#define RX_PW_P0                                0x11
#define RX_PW_P1                                0x12
#define RX_PW_P2                                0x13
#define RX_PW_P3                                0x14
#define RX_PW_P4                                0x15
#define RX_PW_P5                                0x16
#define FIFO_STATUS                             0x17
#define DYNPD                                   0x1C

/* Bit Mnemonics */

/* configuratio nregister */

#define MASK_RX_DR                              6
#define MASK_TX_DS                              5
#define MASK_MAX_RT                             4
#define EN_CRC                                  3
#define CRCO                                    2
#define PWR_UP                                  1
#define PRIM_RX                                 0

/* enable auto acknowledgment */

#define ENAA_P5                                 5
#define ENAA_P4                                 4
#define ENAA_P3                                 3
#define ENAA_P2                                 2
#define ENAA_P1                                 1
#define ENAA_P0                                 0

/* enable rx addresses */

#define ERX_P5                                  5
#define ERX_P4                                  4
#define ERX_P3                                  3
#define ERX_P2                                  2
#define ERX_P1                                  1
#define ERX_P0                                  0

/* setup of address width */

#define AW                                      0       /* 2 bits */

/* setup of auto re-transmission */

#define ARD                                     4       /* 4 bits */
#define ARC                                     0       /* 4 bits */

/* RF setup register */

#define PLL_LOCK                                4
#define RF_DR                                   3
#define RF_PWR                                  1       /* 2 bits */   

/* general status register */

#define RX_DR                                   6
#define TX_DS                                   5
#define MAX_RT                                  4
#define RX_P_NO                                 1       /* 3 bits */
#define TX_FULL                                 0

/* transmit observe register */

#define PLOS_CNT                                4       /* 4 bits */
#define ARC_CNT                                 0       /* 4 bits */

/* fifo status */

#define TX_REUSE                                6
#define FIFO_FULL                               5
#define TX_EMPTY                                4
#define RX_FULL                                 1
#define RX_EMPTY                                0

/* dynamic length */

#define DPL_P0                                  0
#define DPL_P1                                  1
#define DPL_P2                                  2
#define DPL_P3                                  3
#define DPL_P4                                  4
#define DPL_P5                                  5

/* Instruction Mnemonics */

#define R_REGISTER                              0x00    /* last 4 bits will indicate reg. address */
#define W_REGISTER                              0x20    /* last 4 bits will indicate reg. address */
#define REGISTER_MASK                           0x1F
#define R_RX_PAYLOAD                            0x61
#define W_TX_PAYLOAD                            0xA0
#define FLUSH_TX                                0xE1
#define FLUSH_RX                                0xE2
#define REUSE_TX_PL                             0xE3
#define ACTIVATE                                0x50 
#define R_RX_PL_WID                             0x60
#define NOP                                     0xFF

#define nrf24_ADDR_LEN                          5
#define nrf24_CONFIG                            ( ( 1 << EN_CRC ) | ( 0 << CRCO ) )

#define NRF24_TRANSMISSON_OK                    0
#define NRF24_MESSAGE_LOST                      1

/* ---------------------------------------------------------------- VARIABLES */

static uint8_t payload_len;

/* -------------------------------------------- PRIVATE FUNCTION DECLARATIONS */

static uint8_t spi_transfer ( uint8_t tx );

/* --------------------------------------------------------- PUBLIC FUNCTIONS */

/* Driver init */

void bsp_rf_init ( void )
{
    hal_spi_map( &BSP_RF_SPI_MODULE );    
    hal_rf_init( );

    BSP_RF_CE = 0;
    BSP_RF_CS = 1;
}

//  Configure the module.

void bsp_rf_config ( uint8_t channel, uint8_t pay_length )
{
    //  Use static payload length.

    payload_len = pay_length;

    //  Set RF channel.

    bsp_rf_register_config( RF_CH, channel );

    //  Set length of incoming payload.
    
    bsp_rf_register_config( RX_PW_P0, 0x00 );         // Auto-ACK pipe ...
    bsp_rf_register_config( RX_PW_P1, payload_len );  // Data payload pipe
    bsp_rf_register_config( RX_PW_P2, 0x00 );         // Pipe not used 
    bsp_rf_register_config( RX_PW_P3, 0x00 );         // Pipe not used 
    bsp_rf_register_config( RX_PW_P4, 0x00 );         // Pipe not used 
    bsp_rf_register_config( RX_PW_P5, 0x00 );         // Pipe not used 

    //  1 Mbps, TX gain: 0dbm

    bsp_rf_register_config( RF_SETUP, ( 0 << RF_DR ) | ( ( 0x03 ) << RF_PWR ) );

    //  CRC enable, 1 byte CRC length

    bsp_rf_register_config( CONFIG, nrf24_CONFIG );

    //  Auto Acknowledgment.

    bsp_rf_register_config( EN_AA, ( 1 << ENAA_P0 ) | ( 1 << ENAA_P1 ) | ( 0 << ENAA_P2 ) | ( 0 << ENAA_P3 ) | ( 0 << ENAA_P4 ) | ( 0 << ENAA_P5 ) );

    //  Enable RX addresses.

    bsp_rf_register_config( EN_RXADDR, ( 1 << ERX_P0 ) | ( 1 << ERX_P1 ) | ( 0 << ERX_P2 ) | ( 0 << ERX_P3 ) | ( 0 << ERX_P4 ) | ( 0 << ERX_P5 ) );

    //  Auto retransmit delay: 1000 us and Up to 15 retransmit trials.

    bsp_rf_register_config( SETUP_RETR, ( 0x04 << ARD ) | ( 0x0F << ARC ) );

    //  Dynamic length configurations: No dynamic length.

    bsp_rf_register_config( DYNPD, ( 0 << DPL_P0 ) | ( 0 << DPL_P1 ) | ( 0 << DPL_P2 ) | ( 0 << DPL_P3 ) | ( 0 << DPL_P4 ) | ( 0 << DPL_P5 ) );

    // Start listening

    bsp_rf_rx_power_up( );
}

/* Set the RX address */

void bsp_rf_rx_address ( uint8_t * adr )
{
    BSP_RF_CE = 0;
    bsp_rf_register_write( RX_ADDR_P1, adr, nrf24_ADDR_LEN );
    BSP_RF_CE = 1;
}

/* Returns the payload length */

uint8_t bsp_rf_get_payload_length ( void )
{
    return payload_len;
}

/* Set the TX address */

void bsp_rf_tx_address ( uint8_t * adr )
{
    /* RX_ADDR_P0 must be set to the sending addr for auto ack to work. */

    bsp_rf_register_write( RX_ADDR_P0, adr, nrf24_ADDR_LEN );
    bsp_rf_register_write( TX_ADDR, adr, nrf24_ADDR_LEN );
}

/* Checks if data is available for reading */
/* Returns 1 if data is ready ... */

uint8_t bsp_rf_data_ready ( void ) 
{
    // See note in getData() function - just checking RX_DR isn't good enough

    uint8_t status = bsp_rf_get_status( );

    // We can short circuit on RX_DR, but if it's not set, we still need
    // to check the FIFO for any pending packets

    if ( status & ( 1 << RX_DR ) ) 
    {
        return 1;
    }

    return !bsp_rf_rx_fifo_empty( );
}

/* Checks if receive FIFO is empty or not */

uint8_t bsp_rf_rx_fifo_empty ( void )
{
    uint8_t fifoStatus;

    bsp_rf_register_read( FIFO_STATUS, &fifoStatus, 1 );
    
    return ( fifoStatus & ( 1 << RX_EMPTY ) );
}

/* Returns the length of data waiting in the RX fifo */

uint8_t bsp_rf_payload_length ( void )
{
    uint8_t status;

    BSP_RF_CS = 0;
    spi_transfer( R_RX_PL_WID );
    status = spi_transfer( 0x00 );
    BSP_RF_CS = 1;
    
    return status;
}

/* Reads payload bytes into data array */

void bsp_rf_data_read ( uint8_t * data ) 
{
    /* Pull down chip select */

    BSP_RF_CS = 0;

    /* Send cmd to read rx payload */

    spi_transfer( R_RX_PAYLOAD );
    
    /* Read payload */

    bsp_rf_transfer_sync( data, data, payload_len );
    
    /* Pull up chip select */

    BSP_RF_CS = 1;

    /* Reset status register */

    bsp_rf_register_config( STATUS, ( 1 << RX_DR ) );
}

/* Returns the number of retransmissions occured for the last message */

uint8_t bsp_rf_retry_count ( void )
{
    uint8_t rv;

    bsp_rf_register_read( OBSERVE_TX, &rv, 1 );
    rv = rv & 0x0F;

    return rv;
}

// Sends a data package to the default address. Be sure to send the correct
// amount of bytes as configured as payload on the receiver.

void bsp_rf_data_send ( uint8_t * value )
{    
    /* Go to Standby-I first */

    BSP_RF_CE = 0;
     
    /* Set to transmitter mode , Power up if needed */

    bsp_rf_tx_power_up( );

    /* Do we really need to flush TX fifo each time ? */

#if 1
    
    /* Pull down chip select */
    
    BSP_RF_CS = 0;

    /* Write cmd to flush transmit FIFO */

    spi_transfer( FLUSH_TX );

    /* Pull up chip select */

    BSP_RF_CS = 1;

#endif 

    /* Pull down chip select */

    BSP_RF_CS = 0;

    /* Write cmd to write payload */

    spi_transfer( W_TX_PAYLOAD );

    /* Write payload */

    bsp_rf_tx_sync( value, payload_len );

    /* Pull up chip select */

    BSP_RF_CS = 1;

    /* Start the transmission */

    BSP_RF_CE = 1;
}

uint8_t bsp_rf_is_sending ( void )
{
    uint8_t status;

    /* read the current status */

    status = bsp_rf_get_status( );
                
    /* if sending successful (TX_DS) or max retries exceded (MAX_RT). */

    if ( ( status & ( ( 1 << TX_DS ) | ( 1 << MAX_RT ) ) ) )
    { 
        return 0;   /* false */
    }

    return 1;       /* true */

}

uint8_t bsp_rf_get_status ( void )
{
    uint8_t rv;

    BSP_RF_CS = 0;
    rv = spi_transfer( NOP );
    BSP_RF_CS = 1;

    return rv;
}

uint8_t bsp_rf_last_message_status ( void )
{
    uint8_t rv;

    rv = bsp_rf_get_status( );

    if ( ( rv & ( ( 1 << TX_DS ) ) ) )
    {
        /* Transmission went OK */

        return NRF24_TRANSMISSON_OK;
    }
    else if ( ( rv & ( ( 1 << MAX_RT ) ) ) )
    {
        /* Maximum retransmission count is reached */
        /* Last message probably went missing ... */

        return NRF24_MESSAGE_LOST;
    }  
    else
    {
        /* Probably still sending ... */

        return 0xFF;
    }
}

void bsp_rf_rx_power_up ( void )
{     
    BSP_RF_CS = 0;
    spi_transfer( FLUSH_RX );
    BSP_RF_CS = 1;

    bsp_rf_register_config( STATUS, ( 1 << RX_DR ) | ( 1 << TX_DS ) | ( 1 << MAX_RT ) );

    BSP_RF_CE = 0;
    bsp_rf_register_config( CONFIG, nrf24_CONFIG | ( ( 1 << PWR_UP ) | ( 1 << PRIM_RX ) ) );
    BSP_RF_CE = 1;
}

void bsp_rf_tx_power_up ( void )
{
    bsp_rf_register_config( STATUS, ( 1 << RX_DR ) | ( 1 << TX_DS ) | ( 1 << MAX_RT ) );
    bsp_rf_register_config( CONFIG, nrf24_CONFIG | ( ( 1 << PWR_UP ) | ( 0 << PRIM_RX ) ) );
}

void bsp_rf_power_down ( void )
{
    BSP_RF_CE = 0;
    bsp_rf_register_config( CONFIG, nrf24_CONFIG );
}

/* send and receive multiple bytes over SPI */

void bsp_rf_transfer_sync ( uint8_t * dataout, uint8_t * datain, uint8_t len )
{
    uint8_t i;

    for ( i = 0; i < len; i++ )
    {
        datain[i] = spi_transfer( dataout[i] );
    }
}

/* send multiple bytes over SPI */

void bsp_rf_tx_sync ( uint8_t * dataout, uint8_t len )
{
    uint8_t i;
    
    for ( i = 0; i < len; i++ )
    {
        spi_transfer( dataout[i] );
    }
}

/* Clocks only one byte into the given nrf24 register */

void bsp_rf_register_config ( uint8_t reg, uint8_t value )
{
    BSP_RF_CS = 0;
    spi_transfer( W_REGISTER | ( REGISTER_MASK & reg ) );
    spi_transfer( value );
    BSP_RF_CS = 1;
}

/* Read single register from nrf24 */

void bsp_rf_register_read ( uint8_t reg, uint8_t * value, uint8_t len )
{
    BSP_RF_CS = 0;
    spi_transfer( R_REGISTER | ( REGISTER_MASK & reg ) );
    bsp_rf_transfer_sync( value, value, len );
    BSP_RF_CS = 1;
}

/* Write to a single register of nrf24 */

void bsp_rf_register_write ( uint8_t reg, uint8_t * value, uint8_t len )
{
    BSP_RF_CS = 0;
    spi_transfer( W_REGISTER | ( REGISTER_MASK & reg ) );
    bsp_rf_tx_sync( value, len );
    BSP_RF_CS = 1;
}

/* ------------------------------------------- PRIVATE FUNCTION IMPLEMENTATION */

static uint8_t spi_transfer(uint8_t tx)
{
    uint8_t i = 0;
    uint8_t rx = 0;  
    
    hal_spi_transfer( &tx, &rx, 1 );

    return rx;
}

#endif

/* -------------------------------------------------------------------------- */
/*
    bsp_rf_driver.c

  Copyright (c) 2017, MikroElektronika - http://www.mikroe.com

  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgment:
   This product includes software developed by the MikroElektronika.

4. Neither the name of the MikroElektronika nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MIKROELEKTRONIKA ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MIKROELEKTRONIKA BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

----------------------------------------------------------------------------- */