/*
    ethernet_test.h

-----------------------------------------------------------------------------

  This file is part of mikroSDK.
  
  Copyright (c) 2017, MikroElektronika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

/**
        \file   ethernet_test.c

- Version               :   1.0.0
- Date                  :   June 2019
- Developer             :   Milos Vidojevic

*/
/* -------------------------------------------------------------------------- */

#include "test_types.h"
#include "test_console.h"
#include "test_server_page.h"

// Include BSP drivers which will be tested.
//  ...

#include "bsp_ethernet_driver.h"

#if ( _MMB_MODULE_ETHERNET == 1 )

#define TASK_DELAY                      1
#define TASK_NAME                       "ETHERNET Test"

//  Declare necessary initialization variables.
//  ...

static const uint8_t mac_address[6] = { 0x00, 0x14, 0xA5, 0x76, 0x19, 0x3F };

/*
    Device parameters :
        
        - IP Address
        - Gateway Address
        - Network Mask
        - DNS Address
*/
static const uint8_t ip_address[4]  = {  10, 101,  14,  39 };
static const uint8_t gw_address[4]  = {  10, 101,  14,   1 };
static const uint8_t net_mask[4]    = { 255, 255, 255,   0 };
static const uint8_t dns_address[4] = {  10, 101,  14,  99 };

//  ...

void print_ip ( uint8_t * hostname, uint8_t * ip )
{
    test_console_output( hostname, ( void * ) &ip[0], BYTE_FORMAT, "." );
    test_console_output( "", ( void * ) &ip[1], BYTE_FORMAT, "." );
    test_console_output( "", ( void * ) &ip[2], BYTE_FORMAT, "." );
    test_console_output( "", ( void * ) &ip[3], BYTE_FORMAT, "\r\n" );
}

uint16_t Ethernet_Intern_UserTCP ( uint8_t * remoteHost, uint16_t remotePort, uint16_t localPort, uint16_t reqLength, uint8_t * flags ) // TEthInternPktFlags
{
    uint8_t     buf[32];
    uint16_t    len;        //  Reply length.

    //  Listen only to web request on port 80.

    if ( localPort != 80 )
    {
        return 0;
    }

    //  Get 10 first bytes only of the request, the rest does not matter here.

    for ( len = 0; len < 10; len++ )
    {
        buf[len] = Ethernet_Intern_getByte( );
    }

    buf[len] = 0;
    len = 0;

    //  Only GET method supported.

    if ( memcmp( buf, HTTP_GET_METHOD, 5 ) )
    {
        return 0;
    }

    print_ip( "HTTP request from client", remoteHost );

    if ( len == 0 )
    {
        len =  Ethernet_Intern_putConstString( HTTP_HEADER );
        len += Ethernet_Intern_putConstString( HTTP_MIME_HTML_TYPE );
        len += Ethernet_Intern_putConstString( HTTP_WEB_PAGE );
    }

    return len;
}

uint16_t Ethernet_Intern_UserUDP ( uint8_t * remoteHost, uint16_t remotePort, uint16_t destPort, uint16_t reqLength, uint8_t * flags ) // TEthInternPktFlags
{
    uint8_t  buf[32];
    uint16_t len;

    if ( destPort != 10001 )
    {
        return 0;
    }

    //  Reply is made of the remote host IP address in human readable format.

    memset( buf, 0 , 32 );

    ByteToStr( remoteHost[0], buf );        buf[3] = '.';
    ByteToStr( remoteHost[1], buf + 4 );    buf[7] = '.';
    ByteToStr( remoteHost[2], buf + 8 );    buf[11] = '.';
    ByteToStr( remoteHost[3], buf + 12 );

    buf[15] = ':';                       

    //  Then remote host port number.

    WordToStr( remotePort, buf + 16 );      buf[21] = '[';
    WordToStr( destPort, buf + 22 );        buf[27] = ']';

    /*
        The total length of the request is the length of the dynamic string plus 
        the text of the request.
    */

    len = 28 + reqLength;

    //  Puts the dynamic string into the transmit buffer.

    Ethernet_Intern_putBytes( buf, 28 );

    /*  
        Then puts the request string converted into upper char into the transmit 
        buffer.
    */

    while ( reqLength-- )
    {
        char tmp = Ethernet_Intern_getByte( );
        Ethernet_Intern_putByte( toupper( tmp ) );
    }

    return len;           
}

//  ...

void ethernet_test ( test_param_t * tp )
{
    switch ( tp->step ) 
    {
        case TEST_PREINIT :
        {
            tp->step = TEST_INIT;
            tp->delay = TASK_DELAY;
            strcpy( tp->name, TASK_NAME );

            break;
        }
        case TEST_INIT :
        {
            //  Initialize BSP module.
            //  ...

            bsp_ethernet_init( mac_address, ip_address );
            bsp_ethernet_network_init( gw_address, net_mask, dns_address );

            //  Perform all necessary post initialization and configuration actions.
            //  ...



            //  ...
            //  End of module initialization.

            test_console_operation( 1, "Simple Web Server Started" );
            print_ip( "HOST IP", ip_address );

            tp->step = TEST_TASK;

            break;
        }
        case TEST_TASK :
        {
            //  Perform all necessary test tasks.
            //  ...

            Ethernet_Intern_doPacket( );

            //  ...
            //  End of test task.

            break;
        }
        default :
        {
            break;
        }
    }
}

#endif

/* -------------------------------------------------------------------------- */
/*
    ethernet_test.c

  Copyright (c) 2017, MikroElektronika - http://www.mikroe.com

  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgment:
   This product includes software developed by the MikroElektronika.

4. Neither the name of the MikroElektronika nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MIKROELEKTRONIKA ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MIKROELEKTRONIKA BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

----------------------------------------------------------------------------- */